﻿<############################################################################################
#############################################################################################
	"Introduction to Powershell cmdlets for DBAs", script version 1.0
	by Jen McCown, Jen@MidnightDBA.com, http://www.MidnightDBA.com/Jen 
	Session downloads and videos: http://www.MidnightDBA.com/Jen/articles/
    Event videos: http://midnightdba.itbookworm.com/Events

		MidnightSQL Consulting, LLC: 
		When your database needs a hero.

PowershellCmdlets.ps1   Jen McCown     http://www.MidnightDBA.com/Jen

A quick intro: 
     Powershell is everywhere, it's more awesome than you know. It uses short commands, 
     called "commandlets" (cmdlets).
     It also recognizes regular cmd windows commands (DOS) like dir and move and cls.
     Learn to love scripting. And and the get-help cmdlet.

############################################################################################
###########################################################################################>


<##################### CONCEPTS #####################>

<# 
Remember the command prompt?


Run > cmd
You can run commands like: 
    help
    dir
    cd
    echo
    copy

...and other stuff.  Powershell has a similar interface, and in fact, 
recognizes all the command prompt commands!  (They set these up as aliases for cmdlets.)

They set these up as aliases for cmdlets.
#>

echo "hi"

get-alias echo

write-output "hi"

<#
    
Powershell is a scripting language. It's awesome because it's powerful - for example, we can 
choose to run a SQL statement on a list of databases, on a dozen different instances. Or a 
hundred. Whatever we want.  

Powershell is also awesome because it's both scriptable and interactive. You CAN save scripts
and run them - with parameters and includes and loops and IF/THEN logic - or you can just open 
up a window and type commands.

Finally, Powershell is great because they've put a lot of effort into making it EASY. Syntax is 
shorter, help is easy to get, and they've standardized how everything works; if something works
in Exchange, you can do it in IIS or SQL Server...the syntax doesn't change. 

Let's do a quick compare to VBScript. (See CreateFile.vbs and CreateFile.ps1.)

#>


<##################### GETTING STARTED #####################>

    # How cool is this? Powershell treats everything like a directory: SQL Server, registry, etc.
    
    Import-Module SQLPS -DisableNameChecking
    cd sqlserver:

    ###cd HKLM:  Don't play  here
    
    # Let's make something happen
    Write-Host "hi";

<# We used a cmdlet! A cmdlet is literally a little command. #>    

    # Get help about the command
    get-help write-host;
    Get-help write*
    Get-help *sql*
    
    # A cmdlet is essentially an encapsulated .NET class. Get-Help tells us how to CALL a cmdlet.
    #Play with the command!
    write-host "hi" -ForegroundColor Green;
    write-host "hi" -ForegroundColor Red -BackgroundColor White;
    write-host "Hi", "Bye", "Whatever you say", 1 -Separator "---";


    #Let's write to a file instead...
    out-file -InputObject "Hi" -FilePath "c:\temp\myoutfile.txt";
    get-help out-file
    out-file -InputObject "Where are you from?" -FilePath "c:\temp\myoutfile.txt" -Append;


<##################### PIPES AND OBJECTS #####################>

    # About piping things... we're piping a String object to out-file
    "Ooooh, I'm piping things!" | out-file -FilePath "c:\temp\myoutfile.txt" -Append;


  <#  Pipes in Powershell are a bit like SSIS, yeah? You've got the source and destination, and in the
      middle is a pipeline where the data flows you can do multiple things... 
    
      Everything is an OBJECT while it's in the pipeline; it's not just text. The text above is a TEXT OBJECT. 
      If we get a service, it's the actual service.
      
      The pipe is a baton, it allows you to hand off the object you're handling to the next step. 
  #>
    
<##################### VARIABLES #####################>

    # Let's make a variable...
    $txt = "Ooh, variables now!";
    write-host $txt;


    $txt | out-file  -FilePath "c:\temp\myoutfile.txt" -Append;


    $txt = $txt + " For my fellow SQL people.";
    write-host $txt -ForegroundColor Red;


    # Double quotes expand variables; single quotes don't expand the variable:
    write-host "My variable is: $txt" -ForegroundColor Yellow;
    write-host "My variable is: " , $txt

    write-host 'My variable NAME is: $txt' -ForegroundColor Yellow;
    

    #Also cool: you don't have to cast variables, even if they're INT
    $num = 1
    write-host "This is a number: $num" -ForegroundColor Yellow;

        
<##################### GET-MEMBER AND GET-SERVICE #####################>
    # Basically, everything in Powershell is an object.  
    $txt = "Hi"

    # Use Get-Member to find out information about your object: type, methods and properties.
    $txt | Get-Member
    
    write-host $txt.Length

  <# Get-Help is for commands; Get-Member is for objects! #>
  
    $txt.Equals('Hi')
    $txt.Equals('Buttocks')
    $txt.Length

 

    #Let's use Get-Member (or gm for short) on a number variable
    $num = 2
    $num | gm
    
    # Use one of the availables methods...
    $num.Equals(2)


    # Now, let's get the members of another object: a service, from Get-Service!
    Get-Service
    Get-Service *sql*  # Filtered by name
    Get-Service SQLServerAgent

    # Get-MEMBER tells us what properties and methods are available with an object.
    Get-Service SQLServerAgent | Get-Member ;

    # Now we can get more (or less) information out of Get-Service. Choose the column names with select-object
    Get-Service *sql* | Select-Object Name, Status, DisplayName, ServicesDependedOn

    # Same thing, but display it nicely with Format-Table
    Get-Service *sql* | Select-Object Name, Status, DisplayName, ServicesDependedOn | Format-Table -AutoSize
    Get-Service *sql* | Format-Table Name, Status, DisplayName, ServicesDependedOn -AutoSize


    #Is SQL Agent running?
    Get-Service SQLServerAgent
    
    # Stop and start the service. (This must be run in admin mode...)
    (Get-Service SQLServerAgent).Stop() 
    
    
    IF (Get-Service SQLServerAgent).Status -eq "Running") 
        { (Get-Service SQLServerAgent).Stop() 
        }
    (Get-Service SQLServerAgent).Start()


 <##################### GET-CHILDITEM AND FOREACH-OBJECT #####################>

   <# Get-ChildItem #>

    # Let's start by looking at all the items in a directory:
    cd "C:\temp\poshexample";
    dir "C:\temp\poshexample";
    get-alias dir;
    
    Get-ChildItem "C:\temp\poshexample";    # The more Powershelly way...
    
    # Now just the TXT files:
    Get-ChildItem "C:\temp\poshexample" *.txt 



   <# ForEach-Object #>

    # Pass an object, or a list of objects, to ForEach-Object and DO something:
    1, 2, "Heya", $txt | ForEach-Object { write-host $_ -ForegroundColor YELLOW }

   
   <# Now, put your hands together... #>
    Get-ChildItem "C:\temp\poshexample" *.txt | Get-Member

    Get-ChildItem "C:\temp\poshexample" *.txt | ForEach-Object { Write-Host $_.FullName -ForegroundColor Yellow}

    
    Get-ChildItem "C:\temp\poshexample" *.txt | ForEach-Object { notepad $_.FullName }

    Get-ChildItem "C:\temp\poshexample" *.txt | %{ notepad $_.FullName }



    Get-process notepad | Get-Member
    Get-process notepad | %{ $_.Kill() }


    #Remember when we stopped and started a service?
    Get-Service SQLServerAgent | %{ $_.Stop() }
    Get-Service SQLServerAgent | %{ $_.Start() }
    
    
    # Stop the SQL Agent service:
    Get-Service SQLServerAgent | ForEach-Object { $_.Stop() }

    # Stop all the SQL services - bad idea:
    # Get-Service *sql* | %{ $_.Stop() }
    
    # stop all services - DON'T DO THIS!!
    # Get-Service | %{ $_.Stop() }



<##################### POWERSHELL AND SQL #####################>
    #You may have to load SQLPS:
    import-module sqlps;   ## For servers that don’t have it loaded. Like mine.

    # You can run SQL queries!
    # Default to localhost, default db:
    Invoke-Sqlcmd -query "SELECT [Name] as dbname, state_desc from sys.databases WHERE database_id > 4;";  

    get-help Invoke-Sqlcmd 

    #Specify server and database:
    Invoke-Sqlcmd -ServerInstance "YOGACON" -Database "master" -query "SELECT [Name] as dbname, state_desc from sys.databases WHERE database_id > 4;";
    
    # Same thing, with variables: 
    $server = "YOGACON"
    $DB = "master" 
    $query = "SELECT [Name] as dbname, state_desc from sys.databases;" ; 	
    Invoke-Sqlcmd -ServerInstance $server -Database $db -query $query;
    
    # Use input file:
    Invoke-Sqlcmd -ServerInstance "YOGACON" -Database "master" -inputfile "c:\temp\query.sql"



    # Same thing, assigning the result to a variable:
    $query = "SELECT [Name] as dbname, state_desc from sys.databases;" ; 	
    $sql = Invoke-Sqlcmd -query $query;


    #Now we have an array of data records:
    $sql[0]
    $sql[1]
    $sql[2]
    $sql[0].dbname

    $sql | ForEach-Object { Write-Host "My database name is ", $_.dbname }



    
 <##################### BONUS FUN #####################>    

    # Oh yeah, we can run things like calculator, notepad, ssms, MSTSC ...
    calc
    notepad
    notepad "C:\temp\myoutfile.txt"      #We'll talk about how cool this is in a sec....

    # And we can programmatically shut that stuff down:
    Get-Process notepad | stop-process 
    Get-Process -Name calc -ComputerName . | stop-process 


    "Ooh, isn't this interesting?" | out-file -FilePath "c:\temp\myoutfile.txt" -Append
    notepad "c:\temp\myoutfile.txt"
    Get-Process notepad | stop-process 


    # Getting help and exploring
    get-help get*;



<##################### PROGRAMMING STUFF #####################>
    #Arrays and programming constructs
    $array = 1, 2, 3, 4, 5;
    $array[0];
   
    $service = Get-Service SQLServerAgent
    $service[0]
    $service[1]

    #Loops
    $array | ForEach-Object {
        write-host $_;
    }
    
    $array | %{
        write-host $_;
    }    

    

    get-help get-member
    
    #Comparison: Don't use = <> > <, instead use -eq -new -gt -lt
    write-host "1 is equal to 1: ", (1 -eq 1);
    write-host "2 is not equal to 1: ", (2 -ne 1);
    write-host "1 is greater than 1: ",(1 -gt 1);
    

    #Conditionals
    IF (1 -eq 1)
    {
        write-host "1 DOES = 1!!";;
    }
    ELSE
    {
        Write-host "Nothing real is true.";
    }


<##################### RUNNING SCRIPTS #####################>
    get-executionpolicy;
    set-executionpolicy RemoteSigned;     ## You have to run as admin!

    # Execution policies:
    #    Restricted - No scripts can be run. Windows PowerShell can be used only in interactive mode.
    #    AllSigned - Only scripts signed by a trusted publisher can be run.
    #    RemoteSigned - Downloaded scripts must be signed by a trusted publisher before they can be run.
    #    Unrestricted - No restrictions; all Windows PowerShell scripts can be run.


    # Running scripts with .\
    .\myscripts.ps1
    c:\temp\POSH\myscript.ps1

    # Hackers can write functions that sit in memory which intercept commands; Powershell requiring the explicit 
    # folder designaion prevents hackers from stealing the execution call and running their code instead.
